#include <avr/io.h>	
#include "config.h"	
#include <util/delay.h>	

#include "sht11-routines.h"

// Variablen
unsigned char crc_value[2],sht11_state;

// Values
struct {
	uint16_t hum_raw[2];
	uint16_t temp_raw[2];
	int16_t hum[2];
	int16_t temp[2];
} sht11;

// return 0 by sda connection end; return 1 by sda connection
unsigned char	sda_val(char channel)   
{ 
	if(channel == 0)
		return (PIN(SHT11_PORT) & SHT11_SDA) != 0; 
	else if(channel ==1)
		return (PIN(SHT11_PORT) & SHT11_SDA_1) != 0;
		
	return 0;
}

// calculation crc-sum
void crc8(unsigned char b,char channel)
{
	if(channel == 0)
	{
		for (unsigned char i = 0; i < 8; i++) 
		{
			if ((crc_value[0] ^ b) & 0x80) 
			{
				crc_value[0] <<= 1;
				crc_value[0] ^= 0x31;
			} else
	    		crc_value[0] <<= 1;
			b <<= 1;
		}
	}else if (channel == 1)
	{
		for (unsigned char i = 0; i < 8; i++) 
		{
			if ((crc_value[1] ^ b) & 0x80) 
			{
				crc_value[1] <<= 1;
				crc_value[1] ^= 0x31;
			} else
	    		crc_value[1] <<= 1;
			b <<= 1;
		}		
	}
}

/*#####################################
Internal
State Machine Funtions: Mesaure Sequenz
#####################################*/

unsigned char sht11_start_temp(char channel)
{
	if(channel == 0)
	{
		crc_value[0] = SHT11_RESOLUTION << 7; // bit-reversed
		start(0);
		return send(SHT11_CMD_TEMP,0) == 0;
	}else if(channel == 1)
	{
		crc_value[1] = SHT11_RESOLUTION << 7; // bit-reversed
		start(1);
		return send(SHT11_CMD_TEMP,1) == 0;
	}
	
	return 0;
}

unsigned char sht11_start_humid(char channel)
{
	if(channel == 0)
	{
		crc_value[0] = SHT11_RESOLUTION << 7; // bit-reversed
		start(0);
		return send(SHT11_CMD_HUMID,0) == 0;
	}else if(channel == 1)
	{
		crc_value[1] = SHT11_RESOLUTION << 7; // bit-reversed
		start(1);
		return send(SHT11_CMD_HUMID,1) == 0;
	}
	
	return 0;
}

unsigned char sht11_ready(char channel)
{
	if(channel == 0)
		return sda_val(0) == 0;
	else if(channel == 1)
		return sda_val(1) == 0;
		
	return 0;
}

int16_t result(char channel)
{
	int16_t v=0;
	
	if(channel == 0)
	{
		if (!sht11_ready(0))
			return SHT11_UNAVAIL;
		v = recv_data(0) << 8; 
		v |= recv_data(0);
		unsigned char crc = recv_crc(0);
		if (crc != crc_value[0])
			return SHT11_CRC_FAIL;
	}else if(channel == 1)
	{
		if (!sht11_ready(1))
			return SHT11_UNAVAIL;
		v = recv_data(1) << 8; 
		v |= recv_data(1);
		unsigned char crc = recv_crc(1);
		if (crc != crc_value[1])
			return SHT11_CRC_FAIL;
	}
	return v;
}

int16_t sht11_result_temp(char channel)
{
	int16_t v=0;
	if(channel == 0)
	{
		v = result(0);
		if (sht11_valid(v))
		{
			sht11.temp_raw[0]=(uint16_t)v;
			#if SHT11_RESOLUTION
				v = v * 4 - SHT11_TEMP_V_COMP;
			#else
				v -= SHT11_TEMP_V_COMP;
			#endif
    	}
	}else if(channel == 1)
	{
		v = result(1);
		if (sht11_valid(v))
		{
			sht11.temp_raw[1]=(uint16_t)v;
			#if SHT11_RESOLUTION
				v = v * 4 - SHT11_TEMP_V_COMP;
			#else
				v -= SHT11_TEMP_V_COMP;
			#endif
    	}
	}
	return v;
}

int16_t sht11_result_humid(char channel)
{
	int16_t v=0;
	if(channel == 0)
	{
		v = result(0);
		if (sht11_valid(v)) 
		{
			sht11.hum_raw[0]=(uint16_t)v;
			#if SHT11_RESOLUTION
    			// inspired by Werner Hoch, modified for low resolution mode
				const int32_t C1 = (int32_t)(-4.0 * 100);
				const int32_t C2 = (int32_t)(0.648 * 100 * (1L<<24));
				const int32_t C3 = (int32_t)(-7.2e-4 * 100 * (1L<<24));
				v = (int16_t)((((C3 * v + C2) >> 7) * v + (1L<<16)) >> 17) + C1;
 				const int32_t T1 = (uint32_t)(0.01 * (1L<<30));
				const int32_t T2 = (uint32_t)(0.00128 * (1L<<30));
			#else
				// inspired by Werner Hoch
				const int32_t C1 = (int32_t)(-4.0 * 100);
				const int32_t C2 = (int32_t)(0.0405 * 100 * (1L<<28));
				const int32_t C3 = (int32_t)(-2.8e-6 * 100 * (1L<<30));
				v = (int16_t)((((((C3 * v) >> 2) + C2) >> 11) * v + (1L<<16)) >> 17) + C1;
				const int32_t T1 = (uint32_t)(0.01 * (1L<<30));
				const int32_t T2 = (uint32_t)(0.00008 * (1L<<30));
			#endif
			// implemented by whitejack (temp compensation)
			v = (int16_t)( (((((int32_t)sht11.temp[0]-2500) * (int32_t)( (T1+T2*((int32_t)sht11.hum_raw[0])) >>13))>>17) + ((int32_t)v)) );
		}
		if(v>9999)
			v=9999;
		if(v<0001)
			v=0001;
	}else if(channel == 1)
	{
		v = result(1);
		if (sht11_valid(v)) 
		{
			sht11.hum_raw[1]=(uint16_t)v;
			#if SHT11_RESOLUTION
    			// inspired by Werner Hoch, modified for low resolution mode
				const int32_t C1 = (int32_t)(-4.0 * 100);
				const int32_t C2 = (int32_t)(0.648 * 100 * (1L<<24));
				const int32_t C3 = (int32_t)(-7.2e-4 * 100 * (1L<<24));
				v = (int16_t)((((C3 * v + C2) >> 7) * v + (1L<<16)) >> 17) + C1;
 				const int32_t T1 = (uint32_t)(0.01 * (1L<<30));
				const int32_t T2 = (uint32_t)(0.00128 * (1L<<30));
			#else
				// inspired by Werner Hoch
				const int32_t C1 = (int32_t)(-4.0 * 100);
				const int32_t C2 = (int32_t)(0.0405 * 100 * (1L<<28));
				const int32_t C3 = (int32_t)(-2.8e-6 * 100 * (1L<<30));
				v = (int16_t)((((((C3 * v) >> 2) + C2) >> 11) * v + (1L<<16)) >> 17) + C1;
				const int32_t T1 = (uint32_t)(0.01 * (1L<<30));
				const int32_t T2 = (uint32_t)(0.00008 * (1L<<30));
			#endif
			// implemented by whitejack (temp compensation)
			v = (int16_t)( (((((int32_t)sht11.temp[1]-2500) * (int32_t)( (T1+T2*((int32_t)sht11.hum_raw[1])) >>13))>>17) + ((int32_t)v)) );
		}
		if(v>9999)
			v=9999;
		if(v<0001)
			v=0001;
	}

	return v;
}

/*#####################################
Internal
SHT11 Functions
#####################################*/

// reset and start sequence for sht
void start(char channel)
{
	if (channel == 0)
	{
		clrBits(PORT(SHT11_PORT), SHT11_SCK | SHT11_SDA);	// SCK output low, SDA input/high
		setBits(DDR(SHT11_PORT),  SHT11_SCK);
		clrBits(DDR(SHT11_PORT),  SHT11_SDA);
		_delay_us(2);

		// reset communication
		for (unsigned char i = 0; i < 10; i++) 
		{
			sck_pulse;
			_delay_us(2);
		}

		// start sequence
		sck_hi; _delay_us(2);
		sda_lo; _delay_us(2);
		sck_lo; _delay_us(2);
		sck_hi; _delay_us(2);
		sda_hi; _delay_us(2);
		sck_lo; _delay_us(2);
	}
	else if (channel == 1)
	{
		clrBits(PORT(SHT11_PORT), SHT11_SCK_1 | SHT11_SDA_1);	// SCK output low, SDA input/high
		setBits(DDR(SHT11_PORT),  SHT11_SCK_1);
		clrBits(DDR(SHT11_PORT),  SHT11_SDA_1);
		_delay_us(2);

		// reset communication
		for (unsigned char i = 0; i < 10; i++) 
		{
			sck_pulse_1;
			_delay_us(2);
		}

		// start sequence
		sck_hi_1; _delay_us(2);
		sda_lo_1; _delay_us(2);
		sck_lo_1; _delay_us(2);
		sck_hi_1; _delay_us(2);
		sda_hi_1; _delay_us(2);
		sck_lo_1; _delay_us(2);
	}
}

unsigned char send(uint16_t b, char channel)
{
	unsigned char ack=0;
	if(channel == 0)
	{
		crc8(b,0);

		// data
		for (unsigned char i = 0; i < 8; i++) 
		{
			if (b & 0x80)
				sda_hi;
			else
				sda_lo;
			b <<= 1;
			_delay_us(2);
			sck_pulse;
		}

		// acknowledge
		sda_hi;
		_delay_us(2);
		ack = sda_val(0);
		sck_pulse;
	}else if(channel == 1)
	{
		crc8(b,1);

		// data
		for (unsigned char i = 0; i < 8; i++) 
		{
			if (b & 0x80)
				sda_hi_1;
			else
				sda_lo_1;
			b <<= 1;
			_delay_us(2);
			sck_pulse_1;
		}

		// acknowledge
		sda_hi_1;
		_delay_us(2);
		ack = sda_val(1);
		sck_pulse_1;
	}

	return ack;
}

unsigned char recv_data(char channel)
{
	unsigned char b = 0;
	
	if(channel == 0)
	{
		// data
		for (unsigned char i = 0; i < 8; i++) 
		{
			// data is transmitted MSB first
			b <<= 1;
			if (sda_val(0))
	    		b |= 1;
			sck_pulse;
			_delay_us(2);
		}

		// low acknowledge
		sda_lo;
		_delay_us(2);
		sck_pulse;
		sda_hi;
		_delay_us(2);

		crc8(b,0);
	}else if(channel == 1)
	{
		// data
		for (unsigned char i = 0; i < 8; i++) 
		{
			// data is transmitted MSB first
			b <<= 1;
			if (sda_val(1))
	    		b |= 1;
			sck_pulse_1;
			_delay_us(2);
		}

		// low acknowledge
		sda_lo_1;
		_delay_us(2);
		sck_pulse_1;
		sda_hi_1;
		_delay_us(2);

		crc8(b,1);
	}
		
	return b;
}

unsigned char recv_crc(char channel)
{
	unsigned char b = 0;
	
	if(channel == 0)
	{
		// data
		for (unsigned char i = 0; i < 8; i++) 
		{
			// CRC is transmitted LSB first
			b >>= 1;
			if (sda_val(0))
				b |= 0x80;
			sck_pulse;
			_delay_us(2);
		}

		// high acknowledge
		sda_hi;
		_delay_us(2);
		sck_pulse;
		_delay_us(2);
	}else if(channel == 1)
	{
		// data
		for (unsigned char i = 0; i < 8; i++) 
		{
			// CRC is transmitted LSB first
			b >>= 1;
			if (sda_val(1))
				b |= 0x80;
			sck_pulse_1;
			_delay_us(2);
		}

		// high acknowledge
		sda_hi_1;
		_delay_us(2);
		sck_pulse_1;
		_delay_us(2);
	}
	return b;
}

/*#####################################
User Functions
#####################################*/

// Initialisation sht
void sht11_init(void)
{
	start(0);
	send(SHT11_CMD_RESET,0);					// reset
	_delay_ms(11);

	start(0);
	send(SHT11_CMD_WSTAT,0);
	send(SHT11_RESOLUTION,0);					// set resolution

	start(1);
	send(SHT11_CMD_RESET,1);					// reset
	_delay_ms(11);

	start(1);
	send(SHT11_CMD_WSTAT,1);
	send(SHT11_RESOLUTION,1);					// set resolution

	sht11_state=SHT11_STATE_MEASURE_TEMP;
}

void sht11_start_measure(void)
{
	if(sht11_state==SHT11_STATE_READY)
	{
		sht11_state=SHT11_STATE_MEASURE_TEMP;
	}
}

/*#####################################
State Machine:
Init--->sht11_start_measure--->Temp-Measure--->Temp-Calc--->Hum-MMeasure--->Hum-Calc
#####################################*/
unsigned char sht11_measure(void)
{
	switch(sht11_state)
	{
		case SHT11_STATE_MEASURE_TEMP:
		{
			sht11_start_temp(0);
			sht11_state = SHT11_STATE_CALC_TEMP;
		}break;
		
		case SHT11_STATE_CALC_TEMP:
		{
			if(sht11_ready(0))
			{
				sht11.temp[0]=sht11_result_temp(0);
				sht11_state = SHT11_STATE_MEASURE_HUM;
			}
		}break;
		
		case SHT11_STATE_MEASURE_HUM:
		{
			sht11_start_humid(0);
			sht11_state = SHT11_STATE_CALC_HUM;
		}break;
		
		case SHT11_STATE_CALC_HUM:
		{
			if(sht11_ready(0))
			{
				sht11.hum[0]=sht11_result_humid(0);
 				sht11_state = SHT11_STATE_MEASURE_TEMP_1;
			}
		}break;

		case SHT11_STATE_MEASURE_TEMP_1:
		{
			sht11_start_temp(1);
			sht11_state = SHT11_STATE_CALC_TEMP_1;
		}break;

		case SHT11_STATE_CALC_TEMP_1:
		{
			if(sht11_ready(1))
			{
				sht11.temp[1]=sht11_result_temp(1);
				sht11_state = SHT11_STATE_MEASURE_HUM_1;
			}
		}break;
		
		case SHT11_STATE_MEASURE_HUM_1:
		{
			sht11_start_humid(1);
			sht11_state = SHT11_STATE_CALC_HUM_1;
		}break;
		
		case SHT11_STATE_CALC_HUM_1:
		{
			if(sht11_ready(1))
			{
				sht11.hum[1]=sht11_result_humid(1);
 				sht11_state = SHT11_STATE_READY;
			}
		}break;
	}
	return sht11_state;
}

unsigned char sht11_measure_finish(void)
{
	if(sht11_measure()==SHT11_STATE_READY)
		return 1;
	else
		return 0;
}

uint16_t sht11_get_temp_raw(char channel)
{
	if(channel == 0)
		return sht11.temp_raw[0];
	else if(channel == 1)
		return sht11.temp_raw[1];
	
	return 0;
}

uint16_t sht11_get_hum_raw(char channel)
{
	if(channel == 0)
		return sht11.hum_raw[0];
	else if(channel == 1)
		return sht11.hum_raw[1];	
		
	return 0;
}

int16_t sht11_get_temp(char channel)
{
	if(channel == 0)
		return sht11.temp[0];
	else if(channel == 1)
		return sht11.temp[1];
		
	return 0;
}

int16_t sht11_get_hum(char channel)
{
	if(channel == 0)
		return sht11.hum[0];
	else if(channel == 1)
		return sht11.hum[1];
		
	return 0;
}
