#ifndef SHT11_ROUTINES_H
#define SHT11_ROUTINES_H

// Command - Defs
#define SHT11_CMD_TEMP	0x03
#define SHT11_CMD_HUMID	0x05
#define SHT11_CMD_WSTAT	0x06
#define SHT11_CMD_RSTAT	0x07
#define SHT11_CMD_RESET	0x1E

// Comparevalue by Volt
#define SHT11_AT_5V		4010
#define SHT11_AT_4V		3980
#define SHT11_AT_3_5V 	3970
#define SHT11_AT_3V		3960
#define SHT11_AT_2_5V	3940

#define SHT11_TEMP_V_COMP   SHT11_AT_5V

// Resolutionvalue
#define SHT11_RES_LOW		1  // 8_12_Bit
#define SHT11_RES_HIGH		0  // 12_14_Bit
#define SHT11_RESOLUTION	SHT11_RES_HIGH


#define Anz_Sensoren	2			// Anzahl Sensoren
// Connection
#define SHT11_PORT	C
#define SHT11_SCK	(1<<0)				// erster Sensor
#define SHT11_SDA	(1<<1)
#define SHT11_SCK_1	(1<<2)				// zweiter Sensor
#define SHT11_SDA_1	(1<<3)

// Grundmuster
#define	sck_hi setBits(PORT(SHT11_PORT), SHT11_SCK)
#define sck_lo clrBits(PORT(SHT11_PORT), SHT11_SCK)

#define	sck_hi_1 setBits(PORT(SHT11_PORT), SHT11_SCK_1)
#define sck_lo_1 clrBits(PORT(SHT11_PORT), SHT11_SCK_1)

#define sda_hi clrBits(DDR(SHT11_PORT), SHT11_SDA)
#define sda_lo setBits(DDR(SHT11_PORT), SHT11_SDA)

#define sda_hi_1 clrBits(DDR(SHT11_PORT), SHT11_SDA_1)
#define sda_lo_1 setBits(DDR(SHT11_PORT), SHT11_SDA_1)

#define sck_pulse  sck_hi; _delay_us(2); sck_lo
#define sck_pulse_1  sck_hi_1; _delay_us(2); sck_lo_1


// Error
#define		SHT11_UNAVAIL	-32768
#define		SHT11_CRC_FAIL	-32767
#define		sht11_valid(v)	((v) > -32000)

// States
#define		SHT11_STATE_READY			0
#define		SHT11_STATE_MEASURE_TEMP	1
#define		SHT11_STATE_CALC_TEMP		2
#define		SHT11_STATE_MEASURE_HUM		3
#define		SHT11_STATE_CALC_HUM		4
#define		SHT11_STATE_MEASURE_TEMP_1	5
#define		SHT11_STATE_CALC_TEMP_1		6
#define		SHT11_STATE_MEASURE_HUM_1	7
#define		SHT11_STATE_CALC_HUM_1		8

// Functions
// External
// Init
extern void	sht11_init(void);

// call to start measure (starts a sequence to measure and calculate temp and hum)
extern void	sht11_start_measure(void);

// poll to control the measurement an get the values, finish returns 1 else 0 (calls sht11_measure())
extern unsigned char sht11_measure_finish(void);

// get temperature raw value (if measure is finish)
extern uint16_t	sht11_get_temp_raw(char);

// get humidity raw value (if measure is finish)
extern uint16_t	sht11_get_hum_raw(char);

// get temperature value *100 (if measure is finish)
extern int16_t	sht11_get_temp(char);

// get humidity value *100 (if measure is finish)
extern int16_t	sht11_get_hum(char);

// returns and controles the measurement statemachine (called by sht11_measure_finish())
extern unsigned char sht11_measure(void);

// Internal
/*
 * Start measurement (humidity or temperature).
 * Return "device found".
 * Afterwards poll sht11_ready.
 */
unsigned char	sht11_start_temp(char);
unsigned char	sht11_start_humid(char);

// get 0 unless measurement completed.
unsigned char	sht11_ready(char);

// get result of measurement. (H: 100*%RH (0..10000) T: 100*T), get -32xxx on failure.
int16_t	result(char);
int16_t	sht11_result_temp(char);
int16_t	sht11_result_humid(char);

// Return SDA Status
unsigned char sda_val(char);
// crc Berechnung
void  crc8(unsigned char,char);
// Start und Command send
void  start(char);
unsigned char send(uint16_t,char);
unsigned char recv_data(char);
unsigned char recv_crc(char);

#endif // SHT11_ROUTINES_H
