#ifndef UART_ROUTINES_H
	#define UART_ROUTINES_H

	#if defined (__AVR_ATmega128__)
		#define USR UCSR0A
		#define UCR UCSR0B
		#define UDR UDR0
		#define UBRR UBRR0L
		#define USART_RX USART0_RX_vect 
	#endif
	
	#if defined (__AVR_ATmega644__) || defined (__AVR_ATmega644P__)
		#define USR UCSR0A
		#define UCR UCSR0B
		#define UBRR UBRR0L
		#define EICR EICRB
		#define TXEN TXEN0
		#define RXEN RXEN0
		#define RXCIE RXCIE0
		#define UDR UDR0
		#define UDRE UDRE0
		#define USART_RX USART0_RX_vect   
	#endif
	
	#if defined (__AVR_ATmega32__)
		#define USR UCSRA
		#define UCR UCSRB
		#define UBRR UBRRL
		#define EICR EICRB
		#define USART_RX USART_RXC_vect  
	#endif
	
	#if defined (__AVR_ATmega8__)
		#define USR UCSRA
		#define UCR UCSRB
		#define UBRR UBRRL
        #define USART_RX USART_RXC_vect 
	#endif
	
	#if defined (__AVR_ATmega88__)
		#define USR UCSR0A
		#define UCR UCSR0B
		#define UBRR UBRR0L
		#define TXEN TXEN0
		#define UDR UDR0
		#define UDRE UDRE0
		#define USART_RX USART_RXC_vect 
	#endif
	
	#define USART_ECHO	1
	#define BUFFER_SIZE	100	

	volatile unsigned char buffercounter;
	char usart_rx_buffer[BUFFER_SIZE];
	char *rx_buffer_pointer_in;
	char *rx_buffer_pointer_out;

	struct {
			volatile unsigned char usart_ready:1;
			volatile unsigned char usart_rx_ovl:1;
			volatile unsigned char usart_disable:1;
	}usart_status ;	

	#ifndef SYSCLK
		#define SYSCLK F_CPU
	#endif //SYSCLK

	extern void usart_init(unsigned long baudrate); 
	extern void usart_putc(char c);
	extern void usart_puts(char *str);
	extern int usart_getc(void);
	extern void usart_clear(void);

	void usart_put_P (const char *Buffer,...);
	#define usart_put(format, args...)   usart_put_P(PSTR(format) , ## args)

#endif // UART_ROUTINES_H 
